from poium.common import logging

try:
    import uiautomator2 as u2
except ImportError:
    raise ImportError("Please install 'uiautomator2' library")
from poium.config import App


class Android:
    """
    Android driver
    """

    def __init__(self, device_id=None, package_name: str = None):
        self.device_id = device_id
        self.package_name = package_name
        self.driver = None

    def get_device_id(self) -> list:
        """
        get device ID
        """
        import subprocess

        # 执行 adb devices 命令并捕获输出
        result = subprocess.run(['adb', 'devices'], capture_output=True, text=True)

        # 检查命令是否成功执行
        if result.returncode == 0:
            # 如果有输出，处理输出
            devices_info = result.stdout

            # 移除首行的 "List of devices attached"
            devices_info_lines = devices_info.strip().split('\n')[1:]

            # 提取设备信息
            devices = []
            for line in devices_info_lines:
                if line.strip():  # 忽略空行
                    parts = line.split('\t')
                    if len(parts) >= 2:
                        device_id, device_status = parts[:2]
                        devices.append(device_id)

            logging.info(f"📱 device ID:{devices}")
            return devices
        else:
            print("执行 adb devices 失败")

    def connect(self, device_id=None):
        """
        connect device
        """
        if device_id is None and self.device_id is None:
            device_id = self.get_device_id()[0]
        elif device_id is None:
            device_id = self.device_id

        logging.info(f"📱 connect: {device_id}")
        self.driver = u2.connect(device_id)

        return self.driver

    def start_app(self, package_name=None):
        """
        start App
        :param package_name:
        :return:
        """
        if package_name is None:
            package_name = self.package_name

        logging.info(f"📱 start App:{package_name}")
        self.driver.app_start(package_name, use_monkey=True)
        self.driver.app_wait(package_name, front=True, timeout=App.app_wait)
        self.driver.jsonrpc.setConfigurator({"waitForIdleTimeout": 100})

        session = self.driver.session(package_name, attach=True)
        session.implicitly_wait(App.implicitly_wait)
        return session

    def close_app(self, package_name=None):
        """
        close App
        """
        if package_name is None:
            package_name = self.package_name

        logging.info(f"📱 close App:{package_name}")
        self.driver.app_stop(package_name) if package_name is not None else self.driver.app_stop(package_name)



import time
from typing import Union

from poium.common import logging
from poium.common.openatx import BasePage

LOCATOR_LIST = [
    "text",
    "textContains",
    "textMatches",
    "textStartsWith",
    "className",
    "classNameMatches",
    "description",
    "descriptionContains",
    "descriptionMatches",
    "descriptionStartsWith",
    "checkable",
    "checked",
    "clickable",
    "longClickable",
    "scrollable",
    "enabled",
    "focusable",
    "focused",
    "selected",
    "packageName",
    "packageNameMatches",
    "resourceId",
    "resourceIdMatches",
    "index",
    "instance",
]


class Page(BasePage):
    """
    uiautomator2 page class
    """

    def window_size(self):
        """
        Resolution size
        :return: (width, height)
        """
        size = self.driver.window_size()
        logging.info(f"📱 window high: {size[1]} wight: {size[0]}")
        return size

    def app_list(self) -> list:
        """
        Get app list
        :return:
        """
        logging.info(f"📱 app list")
        return self.driver.app_list()

    def app_info(self, package_name=None):
        """
        Get app info
        :param package_name:
        :return:
        """
        if package_name is not None:
            pkg = package_name
        elif self.package_name is not None:
            pkg = self.package_name
        else:
            raise NameError("package name is None")

        logging.info(f"📱 app info: {pkg}")
        return self.driver.app_info(pkg)

    def click(self, x: float = None, y: float = None, text: str = None):
        """
        click position
        :param x:
        :param y:
        :param text:
        :return:
        """
        if not x and not y and not text:
            raise ValueError
        (x, y) = self.driver(text=text).center() if text else (x, y)

        logging.info(f"👆 click: {x}, {y}")

        self.driver.click(x, y)

    def click_more(self, x, y, sleep=0, times=3):
        """
        Multiple clicks.
        :param x:
        :param y:
        :param sleep: interval time
        :param times: click times
        :return:
        """
        w, h = self.window_size()
        x, y = (w * x, h * y) if x < 1 and y < 1 else x, y

        logging.info(f"👆 click {times} times: {x}, {y}")

        for _ in range(times):
            self.driver.touch.down(x, y)
            self.sleep(sleep)
            self.driver.touch.up(x, y)

    def swipe(self, fx: float, fy: float, tx: float, ty: float, duration: float = 0, steps=None, times: int = 1,
              orientation: str = ""):
        """
        UiAutoMatTor use steps instead of duration
          As the document say: Each step execution is throttled to 5ms per step.
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param steps: 1 step is about 5ms, if set, duration will be ignored
        :param times:
        :param orientation:
        :return:
        """
        logging.info(f"👆 {orientation} swipe: [{fx}, {fy}] => [{tx}, {ty}], times: {times} ")
        for _ in range(times):
            self.driver.swipe(fx, fy, tx, ty, duration, steps)
            self.sleep(1)

    def swipe_search(self, text: str, direction: str = "up", x: float = None, y: float = None):
        """
        swipe search text
        :param text:
        :param direction: "down" or "up"
        :param x:
        :param y:
        :return:
        """
        logging.info(f"🔍 swipe search: {text}")
        x = 0.5 if not x else x
        y = 0.5 if not y else y
        for i in range(20):
            if self.driver(textContains=text).exists:
                break
            else:
                if direction == "down":
                    self.swipe_down(fx=x, fy=y, tx=x, ty=y - 0.2)
                elif direction == "up":
                    self.swipe_up(fx=x, fy=y, tx=x, ty=y + 0.2)
                else:
                    raise NameError(f"direction {direction} error.")
        else:
            raise TimeoutError("Timeout, element not found")

    def press(self, key: Union[int, str], meta=None):
        """
        Documents:
            press key via name or key code. Supported key name includes:
            home, back, left, right, up, down, center, menu, search, enter,
            delete(or del), recent(recent apps), volume_up, volume_down,
            volume_mute, camera, power.
        """
        logging.info(f"👆 press: {key}")
        self.driver.press(key, meta=meta)


class XpathElement(object):
    """
    Only CSS selectors are supported.
    https://github.com/openatx/uiautomator2/blob/master/XPATH.md

    >> from poium.u2 import Page, XpathElement
    >> class MyPage(Page):
            input = XpathElement('//android.widget.EditText')
            button = XpathElement('@com.taobao.taobao:id/fl_banner_container')
    """

    def __init__(self, xpath: str, index: int = None, timeout: int = 10, describe: str = None):
        self.xpath = xpath
        self.describe = describe
        self.timeout = timeout
        if index is None:
            self.index = 0
        self.desc = describe

    def __get__(self, instance, owner):
        if instance is None:
            return None
        self.driver = instance.driver
        return self

    def click(self):
        """
        Click element.
        """
        self.driver.xpath(self.xpath).click()
        logging.info(f"✅ Xpath:{self.xpath}, click().")

    def set_text(self, value):
        """
        Simulates typing into the element.
        :param value: input text
        """
        self.driver.xpath(self.xpath).set_text(value)
        logging.info(f"✅ Xpath:{self.xpath}, set_text('{value}').")

    def get_text(self):
        """
        get text from field
        :return
        """
        text = self.driver.xpath(self.xpath).get_text()
        logging.info(f"✅ Xpath:{self.xpath}, get_text().")
        return text

    def match(self):
        """
        None or matched XPathElement
        :return:
        """
        match = self.driver.xpath(self.xpath).match()
        logging.info(f"✅ Xpath:{self.xpath}, match().")
        return match


class Element:
    """
    element class
    """

    def __init__(self, timeout=10, describe=None, **kwargs):
        self.describe = describe
        self.timeout = timeout
        if not kwargs:
            raise ValueError("Please specify a locator")
        self.kwargs = kwargs
        self.k, self.v = next(iter(kwargs.items()))

        if self.k not in LOCATOR_LIST:
            raise KeyError(f"Element positioning of type '{self.k}' is not supported.")

    def __get__(self, instance, owner):
        if instance is None:
            return None
        self.driver = instance.driver
        return self

    def click(self, offset=None):
        """
        Click UI element.

        The click method does the same logic as java uiautomator does.
        1. waitForExists 2. get VisibleBounds center 3. send click event

        :raises:
            UiObjectNotFoundError
        :param offset: (xoff, yoff) default (0.5, 0.5) -> center
        :return:
        """
        logging.info(f"✅ click().")
        self.driver(**self.kwargs).click(self.timeout, offset)

    def click_exists(self) -> bool:
        """
        The element is not clicked until it exists
        """
        is_exists = self.driver(**self.kwargs).click_exists(timeout=self.timeout)
        logging.info(f"✅ click_exists().")
        return is_exists

    def click_more(self, sleep: float = .01, times: int = 3):
        """
        Multiple clicks
        sleep:
        times:
        """
        logging.info(f"✅ click_more(times={times}).")
        x, y = self.center()
        for i in range(times):
            self.driver.touch.down(x, y)
            time.sleep(sleep)
            self.driver.touch.up(x, y)

    def exists(self):
        """
        check if the object exists in current window.
        """
        logging.info(f"✅ exists().")
        is_exists = self.driver(**self.kwargs).exists(timeout=self.timeout)
        return is_exists

    def set_text(self, text: str):
        """
        input text
        :param text:
        """
        logging.info(f"⌨️ set text: {text}.")
        self.driver(**self.kwargs).set_text(text=text)

    def send_keys(self, text: str, clear=True):
        """
        alias of set_text
        :param text:
        :param clear:
        """
        logging.info(f"⌨️ send key: {text}.")
        self.driver(**self.kwargs).click()
        self.driver.send_keys(text=text, clear=clear)

    def clear_text(self):
        """
        Clear the text
        """
        logging.info(f"🧹 clear text.")
        self.driver(**self.kwargs).clear()

    def get_text(self):
        """
        get element text
        """
        text = self.driver(**self.kwargs).get_text()
        logging.info(f"✅ get text: {text}.")
        return text

    def bounds(self):
        """
        Returns the element coordinate position
        :return: left_top_x, left_top_y, right_bottom_x, right_bottom_y
        """
        logging.info(f"✅ bounds().")
        return self.driver(**self.kwargs).bounds()

    def get_position(self):
        """
        get position
        :return: x, y
        """
        logging.info(f"✅ get_position().")
        h, w = self.driver.window_size()
        x, y = self.center()
        return round(x / h, 4), round(y / w, 4)

    def center(self):
        """
        Returns the center coordinates of the element
        return: center point (x, y)
        """
        logging.info(f"✅ center().")
        return self.driver(**self.kwargs).center()

    def swipe(self, direction, times=1, steps=10):
        """
        Performs the swipe action on the UiObject.
        Swipe from center

        :param direction: "left", "right", "up", "down"
        :param times: move times
        :param steps: 1 step is about 5ms, if set, duration will be ignored
        :return:
        """
        assert direction in ("left", "right", "up", "down")
        logging.info(f"👆 {direction} swipe, times: {times}.")
        for i in range(times):
            self.driver(**self.kwargs).swipe(direction=direction, steps=steps)
        time.sleep(0.1)

    def sliding(self, height: float = 0.5):
        """
        Determine if the element is on the current page, and if it isn't, slide down until you find it on the screen
            If present, slide the expected position
        :param height: The screen height 0 ~ 1
        :return:
        """
        logging.info(f"👆 sliding found, height: {height}")
        for i in range(30):
            if self.exists():
                break
            else:
                self.driver.swipe(0.5, 0.7, 0.5, 0.3)
        if height:
            if height <= 0 or height >= 1:
                raise ValueError("'h' checks the range of values, 0 < h < 1")
            else:
                scope = [height - 0.05, height + 0.05]
                for i in range(30):
                    x, y = self.get_position()
                    if scope[0] <= y <= scope[1]:
                        break
                    else:
                        if y > scope[1]:
                            self.driver.swipe(0.5, 0.5, 0.5, 0.45)
                        elif y < scope[0]:
                            self.driver.swipe(0.5, 0.45, 0.5, 0.5)

    @property
    def info(self):
        """
        The element information
        """
        info = self.driver(**self.kwargs).info
        logging.info(f"✅ element info: {info}.")
        return info

    @property
    def count(self):
        """
        Gets the same number of elements
        """
        count = self.driver(**self.kwargs).count
        logging.info(f"✅ count().")
        return count

    def instance(self, num: int = 0):
        """
        Click on the list of elements
        """
        _list = []
        self.wait()
        data = self.count
        for i in range(data):
            _list.append(i)
        element = Element(**self.kwargs, instance=_list[num])
        logging.info(f"✅ click {num} element.")
        return element

    def wait(self):
        """
        Wait until UI Element exists or gone
        """
        logging.info(f"🕣 wait {self.timeout}s.")
        return self.driver(**self.kwargs).wait(exists=True, timeout=self.timeout)

    def wait_gone(self):
        """
        wait until ui gone
        :return:
        """
        logging.info(f"🕣 wait {self.timeout}s gone.")
        return self.driver(**self.kwargs).wait_gone(self.timeout)



import time


class BasePage:
    """
    Define basic generic methods
    u2(uiautomator) and wda(facebook-wda) public base class.
    """

    def __init__(self, driver, package_name: str = None):
        self.driver = driver
        self.package_name = package_name

    @staticmethod
    def sleep(sec) -> None:
        """
        sleep
        :param sec:
        :return:
        """
        time.sleep(sec)

    def swipe_down(self, fx=0.5, fy=0.2, tx=0.5, ty=0.8, duration=0, times=1):
        """
        swipe down
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param times:
        :return:
        """
        self.swipe(fx, fy, tx, ty, duration=duration, times=times, orientation="down")

    def swipe_up(self, fx=0.5, fy=0.8, tx=0.5, ty=0.2, duration=0, times=1):
        """
        swipe up
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param times:
        :return:
        """
        self.swipe(fx, fy, tx, ty, duration=duration, times=times, orientation="up")

    def swipe_left(self, fx=0.8, fy=0.5, tx=0.2, ty=0.5, duration=0, times=1):
        """
        swipe left
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param times:
        :return:
        """
        self.swipe(fx, fy, tx, ty, duration=duration, times=times, orientation="left")

    def swipe_right(self, fx=0.2, fy=0.5, tx=0.8, ty=0.5, duration=0, times=1):
        """
        swipe right
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param times:
        :return:
        """
        self.swipe(fx, fy, tx, ty, duration=duration, times=times, orientation="right")



import sys

from loguru import logger


class Logger:

    def __init__(self, level: str = "DEBUG", colorlog: bool = True):
        self.logger = logger
        self._colorlog = colorlog
        self._console_format = "<green>{time:YYYY-MM-DD HH:mm:ss}</> {file} <level>| {thread.name} | {level} | {message}</level>"
        self._level = level
        self.set_level(self._colorlog, self._console_format, self._level)

    def set_level(self, colorlog: bool = True, format: str = None, level: str = "DEBUG"):
        if format is None:
            format = self._console_format
        try:
            from seldom.running.config import BrowserConfig
            from seldom import Seldom
            if Seldom.driver is not None:
                logfile = BrowserConfig.LOG_PATH
                _file_format = " {time:YYYY-MM-DD HH:mm:ss} | {file: <10} | {thread.name} | {level: <8} | {message}"
                logger.remove()
                logger.add(sys.stderr, level=level, colorize=colorlog, format=format)
                logger.add(logfile, level=level, colorize=colorlog, format=_file_format, encoding="utf-8")
        except ImportError:
            logger.remove()
            logger.add(sys.stderr, level=level, colorize=colorlog, format=format)

    def trace(self, msg: str):
        """
        trace log
        :param msg:
        :return:
        """
        return self.logger.trace(msg)

    def debug(self, msg: str):
        """
        debug log
        :param msg:
        :return:
        """
        return self.logger.debug(msg)

    def info(self, msg: str):
        """
        info log
        :param msg:
        :return:
        """
        self.logger.info(msg)

    def success(self, msg: str):
        """
        success log
        :param msg:
        :return:
        """
        return self.logger.success(msg)

    def warning(self, msg: str):
        """
        warning log
        :param msg:
        :return:
        """
        return self.logger.warning(msg)

    def error(self, msg: str):
        """
        error log
        :param msg:
        :return:
        """
        return self.logger.error(msg)

    def critical(self, msg: str):
        """
        critical log
        :param msg:
        :return:
        """
        return self.logger.critical(msg)


# log level: TRACE < DEBUG < INFO < SUCCESS < WARNING < ERROR
logging = Logger(level="TRACE")



import time

from poium.common import logging
from poium.common.openatx import BasePage

LOCATOR_LIST = [
    "id",
    "name",
    "text",
    "nameContains",
    "label",
    "xpath",
    "labelContains",
    "className",
    "predicate",
    "classChain"
]


class Page(BasePage):
    """
    facebook-wda page class
    """

    def native_resolution(self):
        """
        Gets the original screen resolution.
        """
        multiple = self.driver.scale
        w, h = self.driver.window_size()
        return multiple * w, multiple * h

    def click(self, x: float = None, y: float = None, text: str = None):
        """
        click position
        :param x:
        :param y:
        :param text:
        :return:
        """
        if not x and not y and not text:
            raise ValueError
        (x, y) = self.get_position(text=text) if text else (x, y)

        logging.info(f"👆 click: {x}, {y}")
        self.driver.click(x, y)

    def get_position(self, text=None, element=None):
        """
        Gets element or text position
        :param text:
        :param element:
        :return:
        """
        w, h = self.driver.window_size()
        if text is not None:
            rect = self.driver(name=text).bounds
        elif element is not None:
            rect = element.bounds
        else:
            raise NameError
        x = rect.x + rect.width / 2
        y = rect.y + rect.height / 2
        return x / w, y / h

    def swipe(self, fx: float, fy: float, tx: float, ty: float, duration=0, times=1, orientation: str = ""):
        """
        swipe
        :param fx:
        :param fy:
        :param tx:
        :param ty:
        :param duration:
        :param times:
        :param orientation:
        :return:
        """
        logging.info(f"👆 {orientation} swipe: [{fx}, {fy} =>  {tx}, {ty}], times: {times} ")
        for _ in range(times):
            self.driver.swipe(fx, fy, tx, ty, duration=duration)
            self.sleep(1)

    def swipe_search(self, text, direction="up"):
        """
        swipe search text
        :param text:
        :param direction: "down" or "up"
        :return:
        """
        logging.info(f"🔍 swipe search: {text}")
        for i in range(20):
            if self.driver(text=text).exists and self.driver(text=text).displayed:
                break
            else:
                if direction == "down":
                    self.swipe_down()
                elif direction == "up":
                    self.swipe_up()
                else:
                    raise NameError
        else:
            raise TimeoutError("Timeout, element not found")

    def who_exists(self, element: list = None, text: list = None):
        """
            Determine multiple elements or text for different pages,
        see which comes first, and determine the state of the page
        :param element: Element objects for different pages.
        :param text: text for different pages.
        :return:
            element_child(object): Returns the elements that exist on the current page
            text_child(text): Returns the text that exists on the current page
        """
        for i in range(10):
            if element is not None:
                if type(element) is list:
                    for element_child in element:
                        if element_child.exists() is True:
                            return element_child
                else:
                    raise TypeError("The element must be a list")
            elif text is not None:
                if type(text) is list:
                    for text_child in text:
                        if self.driver(nama=text_child).exists is True:
                            return text_child
                else:
                    raise TypeError("The text must be a list")
            else:
                raise ValueError("Must pass parameter")

            self.sleep(1)

        else:
            raise TypeError("The text or element is not exists")

    def alert(self, click=None, timeout=5) -> bool:
        """
        click error alert.
        :param click:
        :param timeout:
        :return:
        """
        for i in range(timeout):
            if "error" not in self.driver.alert.buttons():
                _list = self.driver.alert.buttons()
                text = self.driver.alert.text
                logging.info(f"alert prompt:⚠ {text}, option button: {_list}")
                if click == "first":
                    logging.info(f"👆 ==> {_list[0]}")
                    self.driver.alert.accept()
                elif click == "second":
                    logging.info(f"👆 ==> {_list[1]}")
                    self.driver.alert.dismiss()
                else:
                    logging.info(f"👆 ==> {click}")
                    self.driver.alert.click(click)
                return True
            else:
                self.sleep(1)
                continue
        else:
            return False


class Element(object):
    """
    element class
    """

    driver = None

    def __init__(self, timeout=10, describe=None, **kwargs):
        self.timeout = timeout
        self.describe = describe
        if not kwargs:
            raise ValueError("Please specify a locator")
        self.kwargs = kwargs
        self.k, self.v = next(iter(kwargs.items()))

        if self.k not in LOCATOR_LIST:
            raise KeyError(f"Element positioning of type '{self.k}' is not supported.")

    def __get__(self, instance, owner):
        if instance is None:
            return None

        self.driver = instance.driver
        return self

    def click(self, focus=None, beyond=None):
        """
        click coordinate position element.
        :param focus: Click the location of the element area, the default click is the center of the element
        :param beyond: Based on the passed element, click on a location other than that element
        :return:
        """
        logging.info(f"✅ click().")
        w, h = self.driver.window_size()
        if self.k == "focus":
            if type(self.v) is not list:
                raise ValueError("The argument must be a list")
            elif self.v[0] > 1 or self.v[1] > 1:
                raise ValueError
            self.driver.click(self.v[0], self.v[1])
        else:
            if focus is not None:
                x, y = self.focus(focus)
            elif beyond is not None:
                xx, yy = self.get_position(percentage=False)
                x, y = xx + beyond[0] * w, yy + beyond[1] * h
            else:
                x, y = self.focus([0.5, 0.5])

            self.driver.click(x / w, y / h)

    def click_exists(self):
        """
        The element is not clicked until it exists
        :return:
        """
        is_exists = self.driver(**self.kwargs).click_exists(self.timeout)
        logging.info(f"✅ click_exists().")
        return is_exists

    def wait(self):
        """
        Wait until UI Element exists
        :return:
        """
        logging.info(f"🕣 wait {self.timeout}s.")
        return self.driver(**self.kwargs).wait(timeout=self.timeout)

    def wait_gone(self):
        """
        wait until ui gone
        :return:
        """
        logging.info(f"🕣 wait {self.timeout}s gone.")
        return self.driver(**self.kwargs).wait_gone(timeout=self.timeout)

    def get(self, raise_error: bool = False):
        """
        Get UI Element
        :param raise_error: whether to raise error if element not found
        :return:
        """
        logging.info(f"✅ get UI element.")
        return self.driver(**self.kwargs).get(timeout=self.timeout, raise_error=raise_error)

    def find_elements(self, text=False):
        """
        查找元素

        Args：
            text(bool): 返回元素对应的文本内容
        """
        logging.info(f"🔍 find elements: {text}.")
        data = self.driver(**self.kwargs).find_elements()
        logging.info(f"I found {len(data)} of them.")

        if text is True:
            text_list = []
            for text_data in data:
                text_list.append(text_data.get_text())
            return text_list
        else:
            return data

    def instance(self, num: int = 0):
        """
        Click on the list of elements
        """
        _list = []
        self.wait()
        data = len(self.find_elements())
        for i in range(data):
            _list.append(i)
        if self.k == "xpath":
            self.kwargs = {"xpath": self.v + "[{}]".format(_list[num])}
            element = Element(**self.kwargs)
        else:
            element = Element(**self.kwargs, index=_list[num])

        logging.info(f"✅ click {num} element.")

        return element

    def set_text(self, text: str):
        """
        input text
        :param text:
        """
        logging.info(f"⌨️ set text: {text}.")
        self.clear_text()
        self.driver(**self.kwargs).set_text(text)

    def clear_text(self):
        """
        Clear the text
        """
        logging.info(f"🧹 clear text.")
        self.driver(**self.kwargs).clear_text()

    def get_text(self):
        """
        获取元素对应的文本
        """
        text = self.driver(**self.kwargs).text
        logging.info(f"✅ get text: {text}.")
        return text

    def swipe(self, direction, times=1, distance=1.0):
        """
        Element-based sliding

        :param direction: "left", "right", "up", "down"
        :param times: move times
        :param distance: 滑动距离
        :return:
        """
        assert direction in ("left", "right", "up", "down")
        logging.info(f"👆 {direction} swipe, times: {times}.")
        for i in range(times):
            self.driver(**self.kwargs).scroll(direction=direction, distance=distance)
        time.sleep(0.1)

    def focus(self, position: list):
        """
        Locate coordinates within the element region
        :param position: Coordinates on the element region
        :return:
        """
        logging.info(f"👆 focus, position: {position}.")
        self.get()
        if type(position) is not list:
            raise NameError("The argument must be a list")
        elif position[0] > 1 or position[1] > 1:
            raise NameError("Coordinates range from 0 to 1")

        rect = self.driver(**self.kwargs).bounds
        x = rect.x + rect.width * position[0]
        y = rect.y + rect.height * position[1]
        return x, y

    def get_position(self, percentage: bool = False):
        """
        Get element coordinates
        :param percentage: True: percentage , False: Default is real coordinates
        :return:
        """

        self.get()
        w, h = self.driver.window_size()
        rect = self.driver(**self.kwargs).bounds
        x = rect.x + rect.width / 2
        y = rect.y + rect.height / 2
        if percentage is True:
            x, y = round(x / w, 6), round(y / h, 6)
        logging.info(f"ℹ️ get element position: [{x}, {y}].")
        return x, y

    def exists(self):
        """
        check if the object exists in current window.
        """
        logging.info(f"✅ exists().")
        if "index" in self.kwargs:
            return True if len(self.find_elements()) > 0 else False
        else:
            return True if self.driver(**self.kwargs).exists and self.driver(**self.kwargs).displayed else False

    def swipe_search(self, direction="up"):
        """
        Scroll to locate the corresponding element
        :param direction: 'down' or 'up'
        :return:
        """
        logging.info(f"🔍 swipe search.")
        for i in range(20):
            if self.exists() is True:
                break
            else:
                if direction == "up":
                    self.driver.swipe(0.5, 0.6, 0.5, 0.4)
                elif direction == "down":
                    self.driver.swipe(0.5, 0.4, 0.5, 0.6)
                else:
                    raise ValueError("The direction parameter can only be 'down' or 'up'")

    def sliding(self, height: float = 0.5):
        """
         Determine if the element is on the current page, and if it isn't, slide down until you find it on the screen
            If present, slide the expected position
        :param height:  The screen height 0 ~ 1
        :return:
        """
        logging.info(f"👆 sliding found, height: {height}")
        if 0 < height < 1:
            height = height
            height_max = height + 0.05
            height_min = height - 0.05
            x, y = self.get_position()
            for i in range(20):
                if height_min <= y <= height_max:
                    break
                move_y = height - y
                if move_y > 0:
                    if move_y >= 0.26:
                        self.driver.swipe(0.5, 0.5, 0.5, 0.6)
                    elif move_y < 0.26:
                        self.driver.swipe(0.5, 0.5, 0.5, 0.52, duration=0.5)
                elif move_y < 0:
                    if move_y <= -0.26:
                        self.driver.swipe(0.5, 0.5, 0.5, 0.4)
                    elif move_y < 0.26:
                        self.driver.swipe(0.5, 0.5, 0.5, 0.48, duration=0.5)
                x, y = self.get_position()

        else:
            raise ValueError("height: The screen height 0 ~ 1")



try:
    import wda
except ImportError:
    raise ImportError("Please install 'facebook-wda' library")

from poium.common import logging

wda.DEBUG = False  # default False
wda.HTTP_TIMEOUT = 180.0  # default 60.0 seconds


class iOS:
    """
    iOS driver
    """

    def __init__(self, device_id=None, package_name: str = None):
        self.device_id = device_id
        self.package_name = package_name
        self.driver = None

    def connect(self, device_id=None):
        """
        connect iOS
        """
        if device_id is None:
            device_id = self.device_id

        self.driver = wda.Client(device_id)
        logging.info(f"📱 connect: {device_id}")

        return self.driver

    def start_app(self, package_name=None):
        """
        start app.
        :param package_name:
        :return:
        """
        if package_name is None:
            package_name = self.package_name

        session = self.driver.session(package_name)
        return session

    def close_app(self):
        """
        close App
        """
        self.driver.close()
